/*
** Codesets.c - Codesets plugin for AmIRC.
** Copyright (C) 2024 Joni Valtanen <jvaltane@kapsi.fi>
*
** Based on Typo.c - Typo plugin for AmIRC.
** Copyright (C) 2002-2003 Robin Cloutman <rycochet@rycochet.com>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**
****************************************************************************
*/

/*
TODO:
  - limit utf8-conversion only for required input. Now touches for all input
  - investigate why ctx->amiplug_out_userwin(...) did not work
  - there are a lot pices of code which are not in use. Remove those

Other:
  - current work is done basically in one night
  - might break some features of AmIRC. Do not blame me yet, this is WIP
*/
#include <exec/types.h>
#include <exec/semaphores.h>
#include <exec/lists.h>
#include <dos/dos.h>
#include <graphics/gfx.h>
#include <clib/alib_protos.h>
#include <proto/exec.h>
#include <proto/dos.h>
#include <proto/intuition.h>
#include <proto/muimaster.h>
#include <proto/codesets.h>
#include <SDI_compiler.h>
#include <SDI_hook.h>
#include "libinit.h"

struct typo
{
	struct typo	*t_next;
	char *t_typo;
};

struct typo_stringnode {
	struct MinNode	n;
	unsigned char	count;
	unsigned char	xor;
	unsigned short	total;
	char *string;
};

#define	BUFFER			2048

//#define	UserData(a)		((struct data*)(a)->userdata)
#define	TypoList(a)		((struct List*)((struct data *)(a)->userdata)->typo_list)

#define	FirstTypo(a)	((struct typo_stringnode*)(((struct List *)a)->lh_Head))
#define	LastTypo(a)		((struct typo_stringnode*)&(((struct List *)a)->lh_Tail))
#define	NextTypo(a)		((struct typo_stringnode*)((a)->n.mln_Succ))

struct data
{
	struct MinList	*typo_list;
	Object			*list;
	Object			*string;
	Object			*add;
	Object			*del;
	Object			*sort;
	char				 buffer[BUFFER];
};

struct wordskip
{
	int	 numwords;	// will skip #channel if second word
	char	*word;
} words[] =
{
	{ 1, "/AWAY" },
	{ 1, "/ME" },
	{ 1, "/QUIT" },
	{ 1, "/TOPIC" },
	{ 1, "/WALLOPS" },
	{ 2, "/DESCRIBE" },
	{ 2, "/KICK" },
	{ 2, "/KICKBAN" },
	{ 2, "/M" },
	{ 2, "/MSG" },
	{ 2, "/NOTICE" },
	{ 2, "/QKICK" },
	{ 2, "/QKICKBAN" },
	{ 2, "/RMSG" },
	{ 2, "/SOUND" },
	{ 0, NULL }
};

// FIXME: move to same place than other related defines
#define COPYRIGHT2 "Copyright " COPYRIGHT "\nGNU General Public License V2"
#define INFO "UTF-8 support using codesets.library"
#define SETUPNAME "Codesets"

//const char *ignore = " ,.!?";	// ";:(){}[]/='";

/* BitPlane Data for "abc" */
/***************************************************************************/

extern struct ExecBase *SysBase;

struct Library *CodesetsBase = NULL;
struct DosLibrary *DOSBase = NULL;
struct SignalSemaphore mySemaphore;
APTR myPool;

struct codeset *amigaCodeset = NULL;
struct codeset *serverCodeset = NULL;

#if 0
struct IntuitionBase *IntuitionBase;
struct Library *MUIMasterBase;
struct MinList typo_list;
#endif

BOOL UserLibOpen( struct myBase *myBase )
{
    // FIXME: use prefs or remove. dos.library is not really used
    DOSBase = (struct DosLibrary *)OpenLibrary("dos.library",37);
    if ( !DOSBase ) return FALSE;
    CodesetsBase = (struct Library *)OpenLibrary(CODESETSNAME, CODESETSVER);
    if ( !CodesetsBase ) return FALSE;
    // FIXME: remove if not used
    myPool = CreatePool(MEMF_CLEAR,4096,1024);
    if ( !myPool )return FALSE;
    InitSemaphore( &mySemaphore );

#if 0
    if ( !(IntuitionBase = (struct IntuitionBase*)OpenLibrary("intuition.library",37)) )return( FALSE );
    if ( !(MUIMasterBase = OpenLibrary("muimaster.library",11)) )return( FALSE );
    NewList( (struct List*)&typo_list );
#endif

    // FIXME: move to prefs section if in future these are editable
    amigaCodeset = CodesetsFindA(NULL, NULL);
    if (!amigaCodeset) {
        return FALSE;
    }

    serverCodeset = CodesetsFind("UTF-8", CSA_FallbackToDefault, FALSE, TAG_DONE);
    if (!serverCodeset) {
        return FALSE;
    }

    return TRUE;
}

void UserLibClose( struct myBase *myBase )
{
#if 0
    CloseLibrary( MUIMasterBase );
    CloseLibrary( (struct Library*)IntuitionBase );
#endif
    DeletePool( myPool );
    if (CodesetsBase) CloseLibrary( (struct Library*)CodesetsBase );
    if (DOSBase) CloseLibrary( (struct Library*)DOSBase );
}

/***************************************************************************/

static inline char	lower	( const char c )							{ return(c>='A'&&c<='Z'?c+'a'-'A':c); }
static inline char	upper	( const char c )							{ return(c>='a'&&c<='z'?c+'A'-'a':c); }
static inline ULONG	max	( ULONG current, ULONG max )			{ return( current>max?max:current ); }

int strlen( const char *a )
{
    int n = 0;
    while( *a++ )n++;
    return(n);
}

// FIXME: Not really strncmp. Rename or fix
int strncmp( const char *a, const char *b, ULONG len )
{
    ULONG pos = 0;
    while( pos < len && *a && *b && *a==*b ) { a++,b++,pos++; };
    if (pos == len - 1) return 0; // match
    return 1; // no match
}


char *strcpy( char *a, const char *b )
{
    while( ( *a++ = *b++ ) );
    return(--a);
}

int stricmp( const char *a, const char *b )
{
    while( *a && *b && lower(*a)==lower(*b) )a++,b++;
    return(lower(*a) - lower(*b));
}

// first word of a, with first word of b
BOOL wordcmp( const char *a, const char *b )
{
    while( *a && *a!=' ' && *b && *b!=' ' && lower(*a) == lower(*b) )a++,b++;
    if ( *a==*b )return( FALSE );
    if ( ( *a==' ' || *b==' ' ) && ( !*a || !*b ) )return( FALSE );
    return( TRUE );
}

// copy first word in b to a, skip spaces to next word in b and return that
char *one_word( char *a, char *b )
{
    while ( *b == ' ' )b++;
    while ( *b && *b != ' ' )*a++ = *b++;
    *a = '\0';
    while ( *b == ' ' )b++;
    return(b);
}

char *skip_word( char *a )
{
    while ( *a == ' ' )a++;
    while ( *a && *a != ' ' )a++;
    while ( *a == ' ' )a++;
    return(a);
}

char *strchr( const char *a, int c )
{
    while ( *a && *a != c )a++;
    if ( !*a )return(NULL);
    return((char*)c);
}

struct typo_stringnode *FindTypo( struct amiplug_functable *ctx, const char *a )
{
    struct typo_stringnode *typo;
    char				*buf = (char*)a;
    unsigned char	 count = 0;
    unsigned char	 xor = 0;
    unsigned short	 total = 0;

    while ( *buf==' ' )buf++;
    while ( *buf && *buf!=' ' )
    {
        count++;
        xor	^= *buf;
        total	+= *buf;
        buf++;
    }
    for ( typo = FirstTypo(ctx) ; typo!=LastTypo(ctx) ; typo = NextTypo(typo) )
    {
        if ( count == typo->count
                && xor == typo->xor
                && total == typo->total
                && !wordcmp(a,typo->string) )
            return( typo );
    }
    return( NULL );
}

void AddTypo( struct amiplug_functable *ctx, const char *a )
{
    struct typo_stringnode *typo;
    char	*buf = (char*)a;

    typo = AllocPooled( myPool, sizeof(struct typo_stringnode) + strlen(a) + 1 );
    typo->count = typo->xor = typo->total = 0;
    strcpy( typo->string, a );
    while ( *buf==' ' )buf++;
    while ( *buf && *buf!=' ' )
    {
        typo->count++;
        typo->xor	^= *buf;
        typo->total	+= *buf;
        buf++;
    }
    AddTail( TypoList(ctx), (struct Node*)typo );
}

void RemTypo( struct amiplug_functable *ctx, struct typo_stringnode *typo )
{
    Remove( (struct Node*)typo );
    FreePooled( myPool, typo, sizeof(struct typo_stringnode) + strlen(typo->string) + 1 );
}

void PurgeTypo( struct amiplug_functable *ctx )
{
    struct typo_stringnode *typo;

    while ( (typo = (struct typo_stringnode*)RemHead(TypoList(ctx)) ) )RemTypo( ctx, typo );
}

/***************************************************************************/

// FIXME: remove if no commands needed
struct amiplug_cmd mycmd1 = {{NULL,NULL},"TYPO","[old [new]]",1,0,FALSE};

struct TagItem mytaglist[] =
{
    { AMIPLUG_Query_MinAmIRCVersion, 0x00030006 },

    { AMIPLUG_Query_Version, VERSION },
    { AMIPLUG_Query_Revision, REVISION },
    { AMIPLUG_Query_Copyright, (ULONG)COPYRIGHT2 },
    { AMIPLUG_Query_Infostring, (ULONG)INFO },

    // FIXME: Clean this when ready
    /*{ AMIPLUG_Query_CustomCommand, (ULONG)0 },*/
    { AMIPLUG_Query_Hook_Input, FALSE },
    { AMIPLUG_Query_Hook_Rawline, TRUE },
    { AMIPLUG_Query_Hook_Rawline2, TRUE },
    { AMIPLUG_Query_Hook_Prefs, FALSE },

    { TAG_DONE }
};

// Setup & Query function
struct TagItem *AMIPLUG_Setup( REG(a0,struct amiplug_functable *ctx) )
{
#if 0
    ctx->userdata = ctx->amiplug_malloc( ctx, sizeof(struct data) );
    ((struct data *)ctx->userdata)->typo_list = &typo_list;
#endif

    return( mytaglist );
}

void AMIPLUG_Cleanup( REG(a0,struct amiplug_functable *ctx) )
{
#if 0
    ctx->amiplug_free( ctx->userdata );
#endif
}

int AMIPLUG_Hook_Rawline( REG(a0,struct amiplug_functable *ctx), REG(a1,STRPTR line), REG(d0,ULONG len) )
{
    int ret = 0;
    //ULONG pos = 0;
    ULONG len2 = 0;
    //STRPTR nick, prefix = "p";
    STRPTR dst = CodesetsConvertStr (CSA_SourceCodeset, serverCodeset,
                                          CSA_DestCodeset, amigaCodeset,
                                          CSA_Source, line,
                                          TAG_DONE);
    if (dst) {
        len2 = strlen (dst);
#if 1
        /* This is kind of dirty trick. Assuption is that UTF8 is always
           longer or same lenght than one-byte per char. -> Just
           overwrite current line
        */
        (void)strcpy (line, dst); /* error handling: no */
#else
        /* Something like this would be probably work better, but...
           For example ctx->amiplug_out_userwin() did nothing with AmIRC 3.6.2
           Probably my mistake with AmIRC header changes. Needs investigation
         */
        nick = dst;
        while (++pos < len2 && dst[pos] != ' ') {};
        pos++;
        PutStr("Server cmd at pos:\"");
        PutStr(&dst[pos]);
        PutStr("\"\n");
        if (pos < len - 1 && !strncmp ("PRIVMSG", &dst[pos], 8)) {
            pos = 0;
            while (++pos < len2 && dst[pos] != '!') {};
            dst[pos] = '\0';
        PutStr("Nick:\"");
        PutStr(nick);
        PutStr("\"\n");
            while (++pos < len2 && dst[pos] != ' ') {}; // cmd
            while (++pos < len2 && dst[pos] != ' ') {}; // receiver
            while (++pos < len2 && dst[pos] != ' ') {}; // message
            pos++;
        PutStr("Message:\"");
        PutStr(&dst[pos]);
        PutStr("\"\n");
            /*if (FALSE == ctx->amiplug_fakerawline(dst, 0)) {
                PutStr("Sending fakerawline failed.\n");
            }*/
            //ctx->amiplug_queueline(ctx, dst);
            ctx->amiplug_out_userwin(ctx, 1, prefix, &dst[pos], nick); // no idea about right color (1)
            ret = EF_Eat;
        }
#endif
        CodesetsFreeA(dst, NULL);
    }
    return ret;
}

int AMIPLUG_Hook_NumericMsg( REG(a0,struct amiplug_functable *ctx), REG(a1,STRPTR line), REG(d0,ULONG numericcode), REG(a2,STRPTR prefix), REG(a3,STRPTR dest), REG(d1,ULONG numargs), REG(a4,STRPTR *args) )
{
    // just a dummy
    return 0 ;
}

void AMIPLUG_DoCommand( REG(a0,struct amiplug_functable *ctx), REG(d0,ULONG commandid), REG(a1,struct amiplug_cmdparm *parms) )
{
    // just a dummy
}

void AMIPLUG_DoMenu( REG(a0,struct amiplug_functable *ctx), REG(d0,ULONG commandid), REG(a1,APTR muiapp) )
{
    // just a dummy
}

void AMIPLUG_Hook_DCC( REG(a0,struct amiplug_functable *ctx), REG(d0,ULONG dccid) )
{
    // just a dummy
}

void AMIPLUG_DoCTCP( REG(a0,struct amiplug_functable *ctx), REG(d0,ULONG commandid), REG(a1,STRPTR from), REG(a2,STRPTR rest), REG(a3,STRPTR to) )
{
    // just a dummy
}

void AMIPLUG_FinalSetup( REG(a0,struct amiplug_functable *ctx) )
{
    // just a dummy
}

char *AMIPLUG_Hook_Input( REG(a0,struct amiplug_functable *ctx), REG(a1,STRPTR string), REG(d0,ULONG len) )
{

#if 0
    /* This is probably not really needed, because servers seems to do
       conversion
    */
    PutStr("Enter\n");
    STRPTR dst = CodesetsConvertStr (CSA_SourceCodeset, amigaCodeset,
                                          CSA_DestCodeset, serverCodeset,
                                          CSA_Source, string,
                                          TAG_DONE);
    PutStr(dst); PutStr("\n");
    if (dst) {
        CodesetsFreeA(dst, NULL);
    }
    PutStr("Leave\n");
#endif

    return string;
}

int AMIPLUG_DoRexx( REG(a0,struct amiplug_functable *ctx), REG(d0,ULONG commandid), REG(a1,STRPTR *args), REG(a2,APTR muiapp) )
{
    // just a dummy
    return 0;
}

void AMIPLUG_Hook_Prefs( REG(a0,struct amiplug_functable *ctx), REG(d0,ULONG methodid), REG(a1,struct amiplug_prefs *prefs) )
{
    // just a dummy
}

int AMIPLUG_Hook_DCCChat( REG(a0,struct amiplug_functable *ctx), REG(d0,ULONG dccid), REG(a1,STRPTR nick), REG(a2,STRPTR line), REG(d1,ULONG len) )
{
    // just a dummy
    return 0;
}

